import os
from tempfile import TemporaryDirectory

from translator_template.app import generate

TEST_DIR = os.path.dirname(os.path.abspath(__file__))


def test_generate():
    with TemporaryDirectory() as dir:
        in_file = os.path.join(TEST_DIR, "data", "test.txt")
        out_file = os.path.join(dir, "test.docx")
        generate(in_file, out_file)



from typing import Iterator, Tuple


def read(f) -> Iterator[Tuple[str, int]]:
    """Reads a text file an produces a stream of tuples representing the page number(s) and frame count(s)"""
    page = 1
    frame = None
    combo = False
    for line_number, line in enumerate(f.readlines()):
        for line_position, c in enumerate(line):
            if c.isspace():
                pass
            elif c == "-":
                yield f"PAGES {page}-{page+1}", frame
                page += 2
                frame = None
            elif c == "*" or c.isdigit():
                if frame is not None and not combo:
                    yield f"PAGE {page}", frame
                    page += 1
                    frame = None
                if c == '*':
                    combo = not combo
                else:
                    frame = (frame or 0) * 10 + int(c)
            else:
                raise RuntimeError(
                    f"Unexpected {c}, line {line_number}, position {line_position}"
                )
    if frame:
        yield f"PAGE {page}", frame



import argparse
import os

from docx import Document
from docx.shared import Inches, Pt
from docx.enum.style import WD_STYLE_TYPE

from .parse import read


def generate(in_file: str, out_file: str) -> None:
    """Read the input specification from the in_file file, and generate a Word document at out_file."""
    document = Document()
    normal_style = document.styles["Normal"]
    normal_style.font.name = "Times New Roman"
    normal_style.font.size = Pt(12)

    def add_style(name: str, indent):
        style = document.styles.add_style(name, WD_STYLE_TYPE.PARAGRAPH)
        style.base_style = normal_style
        style.paragraph_format.left_indent = Inches(indent)
        return style

    page_style = add_style("PagePara", 0)  # noqa(F841)
    panel_style = add_style("PanelPara", 0.5)
    quote_style = add_style("QuotePara", 1.0)

    with open(in_file, "rt") as f:
        for page, panels in read(f):
            document.add_paragraph(f"{page}", style=page_style.name)
            for panel in range(panels):
                document.add_paragraph(f"PANEL {panel+1}", style=panel_style.name)
                document.add_paragraph("", style=quote_style.name)
    document.save(out_file)


def main():
    parser = argparse.ArgumentParser(
        description="Generate a Word Document from page/panel specification"
    )
    parser.add_argument("--in-file", help="input file name", required=True)
    parser.add_argument("--out-file", help="output file name")
    args = parser.parse_args()
    if args.out_file:
        out_file = args.out_file
    else:
        root, _ = os.path.splitext(args.in_file)
        out_file = root + ".docx"
    generate(args.in_file, out_file)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['translator_template', 'translator_template.tests']

package_data = \
{'': ['*'], 'translator_template.tests': ['data/*']}

install_requires = \
['python-docx>=0.8.11,<0.9.0']

entry_points = \
{'console_scripts': ['generate = translator_template.app:main']}

setup_kwargs = {
    'name': 'translator-template',
    'version': '0.1.9',
    'description': 'Utility for generating a Microsoft Word file from a simple document description, for translators.',
    'long_description': '# translator-template\nGenerate a Word Document Template for Translator\n\n## Prerequisite\n\n* python 3.7 or greater\n* poetry\n\nDownload the dependent modules by running `poetry install`\n\n## Running the code\n\n```\npoetry run generate --in-file {input-text-file} --out-file {output-docx-file}\n```\n\nIf the `--out-file` argument is omitted, the `--in-file` argument value is used, with the extension replaced with `.docx`.\n\n## Input Format Specification\n\nIn regular expression form the input is the following:\n\n`^((\\d|\\*\\d+\\*)-?)*$`\n\nFor a given page, the number of panels is represented either by a single digit if the count of panels is fewer than 10.  When the count of panels is greater than or equal to 9, the count is sandwiched between a pair of asterisks.\n\nThe panel count can be optionally followed by a hyphen.  This signifies that the panels span two pages.\n\n## Output Specification\n\nThe output is a Word DOCX file.  This contains three paragraph styles:\n\n* PagePara\n* PanelPara\n* QuotePara\n',
    'author': 'Chris Thrasher',
    'author_email': 'cthrash@github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from io import StringIO

from translator_template.parse import read


def test_read():
    def read_from(s):
        with StringIO(s) as f:
            return [p for p in read(f)]

    assert [("PAGE 1", 2)] == read_from("2")
    assert [("PAGE 1", 2), ("PAGE 2", 3)] == read_from("23")
    assert [("PAGE 1", 2), ("PAGES 2-3", 3)] == read_from("23-")
    assert [("PAGE 1", 56)] == read_from("*56*")
    assert [("PAGES 1-2", 56), ("PAGE 3", 4)] == read_from("*56*-4")
    assert [("PAGE 1", 1), ("PAGE 2", 0), ("PAGE 3", 1)] == read_from("101")
    assert [("PAGE 1", 1), ("PAGE 2", 23), ("PAGE 3", 4)] == read_from("1*23*4")
    assert [("PAGE 1", 1), ("PAGE 2", 23), ("PAGE 3", 45)] == read_from("1*23**45*")
    assert [("PAGE 1", 1), ("PAGES 2-3", 23), ("PAGE 4", 45)] == read_from("1*23*-*45*")
